@@ -36,13 +36,46 @@
 
 #endif  /* NOTDEF */
 
+struct pt_load_segment {
+	off_t file_offset;
+	physaddr_t phys_start;
+	physaddr_t phys_end;
+};
+
+struct vmcore_data {
+	ulong flags;
+	int ndfd;
+	FILE *ofp;
+	uint header_size;
+	char *elf_header;
+	uint num_pt_load_segments;
+	struct pt_load_segment *pt_load_segments;
+        Elf32_Ehdr *elf32;
+        Elf32_Phdr *notes32;
+        Elf32_Phdr *load32;
+        Elf64_Ehdr *elf64;
+        Elf64_Phdr *notes64;
+        Elf64_Phdr *load64;
+        void *nt_prstatus;
+        void *nt_prpsinfo;
+        void *nt_taskstruct;
+	ulong task_struct;
+	uint page_size;
+	ulong switch_stack;
+	uint num_prstatus_notes;
+	void *nt_prstatus_percpu[NR_CPUS];
+	struct xen_kdump_data *xen_kdump_data;
+};
+
 /*
  *  Proposed ELF note for Xen (writable pagetable) per-vcpu CR3.
  */
 #define NT_XEN_KDUMP_CR3 0x10000001
 
 struct xen_kdump_data {
+	ulong flags;
 	ulong cr3;
+	ulong p2m_mfn;
 	char *page;
 	ulong last_mfn_read;
 	ulong cache_hits;
@@ -51,4 +84,8 @@ struct xen_kdump_data {
         ulong *p2m_mfn_frame_list;
 };
 
+#define KDUMP_P2M_INIT  (0x1)
+#define KDUMP_CR3       (0x2)
+#define KDUMP_MFN_LIST  (0x4)
+
 #define P2M_FAILURE ((physaddr_t)(0xffffffffffffffffLL))
